@@ -33,7 +33,7 @@ try:
                            OneOrMore, Combine, Or, Optional, Suppress, Group,
                            nums, alphanums, printables,
                            stringEnd, pythonStyleComment,
-                           ParseBaseException)
+                           ParseBaseException, __diag__)
 except ImportError:
     print('pyparsing is not available')
     sys.exit(77)
@@ -50,6 +50,12 @@ except ImportError:
     # don't do caching on old python
     lru_cache = lambda: (lambda f: f)
 
+__diag__.warn_multiple_tokens_in_named_alternation = True
+__diag__.warn_ungrouped_named_tokens_in_collection = True
+__diag__.warn_name_set_on_empty_Forward = True
+__diag__.warn_on_multiple_string_args_to_oneof = True
+__diag__.enable_debug_on_named_expressions = True
+
 EOL = LineEnd().suppress()
 EMPTYLINE = LineEnd()
 COMMENTLINE = pythonStyleComment + EOL
@@ -111,9 +117,9 @@ def hwdb_grammar():
 def property_grammar():
     ParserElement.setDefaultWhitespaceChars(' ')
 
-    dpi_setting = (Optional('*')('DEFAULT') + INTEGER('DPI') + Suppress('@') + INTEGER('HZ'))('SETTINGS*')
+    dpi_setting = Group(Optional('*')('DEFAULT') + INTEGER('DPI') + Suppress('@') + INTEGER('HZ'))('SETTINGS*')
     mount_matrix_row = SIGNED_REAL + ',' + SIGNED_REAL + ',' + SIGNED_REAL
-    mount_matrix = (mount_matrix_row + ';' + mount_matrix_row + ';' + mount_matrix_row)('MOUNT_MATRIX')
+    mount_matrix = Group(mount_matrix_row + ';' + mount_matrix_row + ';' + mount_matrix_row)('MOUNT_MATRIX')
     xkb_setting = Optional(Word(alphanums + '+-/@._'))
 
     props = (('MOUSE_DPI', Group(OneOrMore(dpi_setting))),
@@ -121,20 +127,20 @@ def property_grammar():
              ('MOUSE_WHEEL_CLICK_ANGLE_HORIZONTAL', INTEGER),
              ('MOUSE_WHEEL_CLICK_COUNT', INTEGER),
              ('MOUSE_WHEEL_CLICK_COUNT_HORIZONTAL', INTEGER),
-             ('ID_AUTOSUSPEND', Literal('1')),
-             ('ID_INPUT', Literal('1')),
-             ('ID_INPUT_ACCELEROMETER', Literal('1')),
-             ('ID_INPUT_JOYSTICK', Literal('1')),
-             ('ID_INPUT_KEY', Literal('1')),
-             ('ID_INPUT_KEYBOARD', Literal('1')),
-             ('ID_INPUT_MOUSE', Literal('1')),
-             ('ID_INPUT_POINTINGSTICK', Literal('1')),
-             ('ID_INPUT_SWITCH', Literal('1')),
-             ('ID_INPUT_TABLET', Literal('1')),
-             ('ID_INPUT_TABLET_PAD', Literal('1')),
-             ('ID_INPUT_TOUCHPAD', Literal('1')),
-             ('ID_INPUT_TOUCHSCREEN', Literal('1')),
-             ('ID_INPUT_TRACKBALL', Literal('1')),
+             ('ID_AUTOSUSPEND', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_ACCELEROMETER', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_JOYSTICK', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_KEY', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_KEYBOARD', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_MOUSE', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_POINTINGSTICK', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_SWITCH', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_TABLET', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_TABLET_PAD', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_TOUCHPAD', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_TOUCHSCREEN', Or((Literal('0'), Literal('1')))),
+             ('ID_INPUT_TRACKBALL', Or((Literal('0'), Literal('1')))),
              ('POINTINGSTICK_SENSITIVITY', INTEGER),
              ('POINTINGSTICK_CONST_ACCEL', REAL),
              ('ID_INPUT_JOYSTICK_INTEGRATION', Or(('internal', 'external'))),
@@ -240,10 +246,19 @@ def check_one_keycode(prop, value):
                 'KBD_LCD_MENU' in key):
             error('Keycode {} unknown', key)
 
+def check_wheel_clicks(properties):
+    pairs = (('MOUSE_WHEEL_CLICK_COUNT_HORIZONTAL', 'MOUSE_WHEEL_CLICK_COUNT'),
+             ('MOUSE_WHEEL_CLICK_ANGLE_HORIZONTAL', 'MOUSE_WHEEL_CLICK_ANGLE'),
+             ('MOUSE_WHEEL_CLICK_COUNT_HORIZONTAL', 'MOUSE_WHEEL_CLICK_ANGLE_HORIZONTAL'),
+             ('MOUSE_WHEEL_CLICK_COUNT', 'MOUSE_WHEEL_CLICK_ANGLE'))
+    for pair in pairs:
+        if pair[0] in properties and pair[1] not in properties:
+            error('{} requires {} to be specified', *pair)
+
 def check_properties(groups):
     grammar = property_grammar()
     for matches, props in groups:
-        prop_names = set()
+        seen_props = {}
         for prop in props:
             # print('--', prop)
             prop = prop.partition('#')[0].rstrip()
@@ -253,9 +268,9 @@ def check_properties(groups):
                 error('Failed to parse: {!r}', prop)
                 continue
             # print('{!r}'.format(parsed))
-            if parsed.NAME in prop_names:
+            if parsed.NAME in seen_props:
                 error('Property {} is duplicated', parsed.NAME)
-            prop_names.add(parsed.NAME)
+            seen_props[parsed.NAME] = parsed.VALUE
             if parsed.NAME == 'MOUSE_DPI':
                 check_one_default(prop, parsed.VALUE.SETTINGS)
             elif parsed.NAME == 'ACCEL_MOUNT_MATRIX':
@@ -264,6 +279,8 @@ def check_properties(groups):
                 val = parsed.VALUE if isinstance(parsed.VALUE, str) else parsed.VALUE[0]
                 check_one_keycode(prop, val)
 
+        check_wheel_clicks(seen_props)
+
 def print_summary(fname, groups):
     n_matches = sum(len(matches) for matches, props in groups)
     n_props = sum(len(props) for matches, props in groups)
