@@ -3,6 +3,7 @@
 
 #include <stdbool.h>
 
+#include "locale-util.h"
 #include "macro.h"
 
 /* The enum order is used to order unit jobs in the job queue
@@ -21,7 +22,7 @@ typedef enum UnitType {
         UNIT_SLICE,
         UNIT_SCOPE,
         _UNIT_TYPE_MAX,
-        _UNIT_TYPE_INVALID = -1
+        _UNIT_TYPE_INVALID = -EINVAL,
 } UnitType;
 
 typedef enum UnitLoadState {
@@ -33,7 +34,7 @@ typedef enum UnitLoadState {
         UNIT_MERGED,
         UNIT_MASKED,
         _UNIT_LOAD_STATE_MAX,
-        _UNIT_LOAD_STATE_INVALID = -1
+        _UNIT_LOAD_STATE_INVALID = -EINVAL,
 } UnitLoadState;
 
 typedef enum UnitActiveState {
@@ -45,7 +46,7 @@ typedef enum UnitActiveState {
         UNIT_DEACTIVATING,
         UNIT_MAINTENANCE,
         _UNIT_ACTIVE_STATE_MAX,
-        _UNIT_ACTIVE_STATE_INVALID = -1
+        _UNIT_ACTIVE_STATE_INVALID = -EINVAL,
 } UnitActiveState;
 
 typedef enum FreezerState {
@@ -54,16 +55,23 @@ typedef enum FreezerState {
         FREEZER_FROZEN,
         FREEZER_THAWING,
         _FREEZER_STATE_MAX,
-        _FREEZER_STATE_INVALID = -1
+        _FREEZER_STATE_INVALID = -EINVAL,
 } FreezerState;
 
+typedef enum UnitMarker {
+        UNIT_MARKER_NEEDS_RELOAD,
+        UNIT_MARKER_NEEDS_RESTART,
+        _UNIT_MARKER_MAX,
+        _UNIT_MARKER_INVALID = -EINVAL,
+} UnitMarker;
+
 typedef enum AutomountState {
         AUTOMOUNT_DEAD,
         AUTOMOUNT_WAITING,
         AUTOMOUNT_RUNNING,
         AUTOMOUNT_FAILED,
         _AUTOMOUNT_STATE_MAX,
-        _AUTOMOUNT_STATE_INVALID = -1
+        _AUTOMOUNT_STATE_INVALID = -EINVAL,
 } AutomountState;
 
 /* We simply watch devices, we cannot plug/unplug them. That
@@ -73,7 +81,7 @@ typedef enum DeviceState {
         DEVICE_TENTATIVE, /* mounted or swapped, but not (yet) announced by udev */
         DEVICE_PLUGGED,   /* announced by udev */
         _DEVICE_STATE_MAX,
-        _DEVICE_STATE_INVALID = -1
+        _DEVICE_STATE_INVALID = -EINVAL,
 } DeviceState;
 
 typedef enum MountState {
@@ -90,7 +98,7 @@ typedef enum MountState {
         MOUNT_FAILED,
         MOUNT_CLEANING,
         _MOUNT_STATE_MAX,
-        _MOUNT_STATE_INVALID = -1
+        _MOUNT_STATE_INVALID = -EINVAL,
 } MountState;
 
 typedef enum PathState {
@@ -99,7 +107,7 @@ typedef enum PathState {
         PATH_RUNNING,
         PATH_FAILED,
         _PATH_STATE_MAX,
-        _PATH_STATE_INVALID = -1
+        _PATH_STATE_INVALID = -EINVAL,
 } PathState;
 
 typedef enum ScopeState {
@@ -110,7 +118,7 @@ typedef enum ScopeState {
         SCOPE_STOP_SIGKILL,
         SCOPE_FAILED,
         _SCOPE_STATE_MAX,
-        _SCOPE_STATE_INVALID = -1
+        _SCOPE_STATE_INVALID = -EINVAL,
 } ScopeState;
 
 typedef enum ServiceState {
@@ -134,14 +142,14 @@ typedef enum ServiceState {
         SERVICE_AUTO_RESTART,
         SERVICE_CLEANING,
         _SERVICE_STATE_MAX,
-        _SERVICE_STATE_INVALID = -1
+        _SERVICE_STATE_INVALID = -EINVAL,
 } ServiceState;
 
 typedef enum SliceState {
         SLICE_DEAD,
         SLICE_ACTIVE,
         _SLICE_STATE_MAX,
-        _SLICE_STATE_INVALID = -1
+        _SLICE_STATE_INVALID = -EINVAL,
 } SliceState;
 
 typedef enum SocketState {
@@ -160,7 +168,7 @@ typedef enum SocketState {
         SOCKET_FAILED,
         SOCKET_CLEANING,
         _SOCKET_STATE_MAX,
-        _SOCKET_STATE_INVALID = -1
+        _SOCKET_STATE_INVALID = -EINVAL,
 } SocketState;
 
 typedef enum SwapState {
@@ -174,14 +182,14 @@ typedef enum SwapState {
         SWAP_FAILED,
         SWAP_CLEANING,
         _SWAP_STATE_MAX,
-        _SWAP_STATE_INVALID = -1
+        _SWAP_STATE_INVALID = -EINVAL,
 } SwapState;
 
 typedef enum TargetState {
         TARGET_DEAD,
         TARGET_ACTIVE,
         _TARGET_STATE_MAX,
-        _TARGET_STATE_INVALID = -1
+        _TARGET_STATE_INVALID = -EINVAL,
 } TargetState;
 
 typedef enum TimerState {
@@ -191,7 +199,7 @@ typedef enum TimerState {
         TIMER_ELAPSED,
         TIMER_FAILED,
         _TIMER_STATE_MAX,
-        _TIMER_STATE_INVALID = -1
+        _TIMER_STATE_INVALID = -EINVAL,
 } TimerState;
 
 typedef enum UnitDependency {
@@ -236,7 +244,7 @@ typedef enum UnitDependency {
         UNIT_REFERENCED_BY,
 
         _UNIT_DEPENDENCY_MAX,
-        _UNIT_DEPENDENCY_INVALID = -1
+        _UNIT_DEPENDENCY_INVALID = -EINVAL,
 } UnitDependency;
 
 typedef enum NotifyAccess {
@@ -245,7 +253,7 @@ typedef enum NotifyAccess {
         NOTIFY_MAIN,
         NOTIFY_EXEC,
         _NOTIFY_ACCESS_MAX,
-        _NOTIFY_ACCESS_INVALID = -1
+        _NOTIFY_ACCESS_INVALID = -EINVAL,
 } NotifyAccess;
 
 char *unit_dbus_path_from_name(const char *name);
@@ -266,6 +274,9 @@ UnitActiveState unit_active_state_from_string(const char *s) _pure_;
 const char *freezer_state_to_string(FreezerState i) _const_;
 FreezerState freezer_state_from_string(const char *s) _pure_;
 
+const char *unit_marker_to_string(UnitMarker m) _const_;
+UnitMarker unit_marker_from_string(const char *s) _pure_;
+
 const char* automount_state_to_string(AutomountState i) _const_;
 AutomountState automount_state_from_string(const char *s) _pure_;
 
@@ -304,3 +315,5 @@ UnitDependency unit_dependency_from_string(const char *s) _pure_;
 
 const char* notify_access_to_string(NotifyAccess i) _const_;
 NotifyAccess notify_access_from_string(const char *s) _pure_;
+
+SpecialGlyph unit_active_state_to_glyph(UnitActiveState state);
